@@ -183,13 +183,22 @@ def get_meta_from_archive(path):
                 for key in ('Name', 'Version', 'Description', 'Summary')]
 
 
+def cleanup(name, sep="-"):
+    """Used for sanitizing addon names. The function removes Orange/Orange3
+    from the name and adds spaces before upper letters of the leftover to
+    separate its words."""
+    prefix, separator, postfix = name.partition(sep)
+    name = postfix if separator == sep else prefix
+    return " ".join(re.findall("[A-Z][a-z]*", name[0].upper() + name[1:]))
+
+
 class AddonManagerWidget(QWidget):
 
     statechanged = Signal()
 
     def __init__(self, parent=None, **kwargs):
         super(AddonManagerWidget, self).__init__(parent, **kwargs)
-
+        self.__items = []
         self.setLayout(QVBoxLayout())
 
         self.__header = QLabel(
@@ -277,7 +286,7 @@ class AddonManagerWidget(QWidget):
             else:
                 item1.setCheckState(Qt.Unchecked)
 
-            item2 = QStandardItem(name)
+            item2 = QStandardItem(cleanup(name))
 
             item2.setFlags(Qt.ItemIsEnabled | Qt.ItemIsSelectable)
             item2.setToolTip(summary)
@@ -651,7 +660,7 @@ def list_pypi_addons():
 
     pypi = xmlrpc.client.ServerProxy(
         "https://pypi.python.org/pypi/",
-        transport=xmlrpc.client.SafeTransport()
+        transport=SafeUrllibTransport()
     )
     addons = pypi.search(ADDON_PYPI_SEARCH_SPEC)
 
