@@ -11,25 +11,38 @@ from io import BytesIO, StringIO
 
 import pkg_resources
 
-from PyQt4.QtGui import (
+from AnyQt.QtWidgets import (
     QMainWindow, QWidget, QAction, QActionGroup, QMenu, QMenuBar, QDialog,
-    QFileDialog, QMessageBox, QVBoxLayout, QSizePolicy, QColor, QKeySequence,
-    QIcon, QToolBar, QToolButton, QDockWidget, QDesktopServices, QApplication,
-    QShortcut
+    QFileDialog, QMessageBox, QVBoxLayout, QSizePolicy, QToolBar, QToolButton,
+    QDockWidget, QApplication, QShortcut
+)
+from AnyQt.QtGui import QColor, QIcon, QDesktopServices, QKeySequence
+
+from AnyQt.QtCore import (
+    Qt, QEvent, QSize, QUrl, QTimer, QFile, QByteArray, QSettings, QT_VERSION
 )
 
-from PyQt4.QtCore import (
-    Qt, QEvent, QSize, QUrl, QTimer, QFile, QByteArray
-)
+try:
+    from AnyQt.QtWebEngineWidgets import QWebEngineView
+    USE_WEB_ENGINE = True
+except ImportError:
+    from AnyQt.QtWebKitWidgets import QWebView
+    from AnyQt.QtNetwork import QNetworkDiskCache
+    USE_WEB_ENGINE = False
 
-from PyQt4.QtNetwork import QNetworkDiskCache
 
-from PyQt4.QtWebKit import QWebView
+from AnyQt.QtCore import pyqtProperty as Property
 
-from PyQt4.QtCore import pyqtProperty as Property
-
-# Compatibility with PyQt < v4.8.3
-from ..utils.qtcompat import QSettings
+if QT_VERSION >= 0x50000:
+    from AnyQt.QtCore import QStandardPaths
+    def user_documents_path():
+        """Return the users 'Documents' folder path."""
+        return QStandardPaths.writableLocation(
+            QStandardPaths.DocumentsLocation)
+else:
+    def user_documents_path():
+        return QDesktopServices.storageLocation(
+            QDesktopServices.DocumentsLocation)
 
 from ..gui.dropshadow import DropShadowFrame
 from ..gui.dock import CollapsibleDockWidget
@@ -155,9 +168,7 @@ class CanvasMainWindow(QMainWindow):
 
         self.widget_registry = None
 
-        self.last_scheme_dir = QDesktopServices.StandardLocation(
-            QDesktopServices.DocumentsLocation
-        )
+        self.last_scheme_dir = user_documents_path()
         try:
             self.recent_schemes = config.recent_schemes()
         except Exception:
@@ -334,13 +345,17 @@ class CanvasMainWindow(QMainWindow):
                                     objectName="help-dock",
                                     allowedAreas=Qt.RightDockWidgetArea |
                                                  Qt.BottomDockWidgetArea)
-        self.help_view = QWebView()
-        manager = self.help_view.page().networkAccessManager()
-        cache = QNetworkDiskCache()
-        cache.setCacheDirectory(
-            os.path.join(config.cache_dir(), "help", "help-view-cache")
-        )
-        manager.setCache(cache)
+        self.help_dock.setAllowedAreas(Qt.NoDockWidgetArea)
+        if USE_WEB_ENGINE:
+            self.help_view = QWebEngineView()
+        else:
+            self.help_view = QWebView()
+            manager = self.help_view.page().networkAccessManager()
+            cache = QNetworkDiskCache()
+            cache.setCacheDirectory(
+                os.path.join(config.cache_dir(), "help", "help-view-cache")
+            )
+            manager.setCache(cache)
         self.help_dock.setWidget(self.help_view)
         self.addDockWidget(
             QSettings().value('help-dock/area', Qt.RightDockWidgetArea, type=int),
@@ -567,7 +582,10 @@ class CanvasMainWindow(QMainWindow):
                     triggered=self.reset_widget_settings)
 
     def setup_menu(self):
-        menu_bar = QMenuBar()
+        if sys.platform == "darwin" and QT_VERSION >= 0x50000:
+            self.__menu_glob = QMenuBar(None)
+
+        menu_bar = QMenuBar(self)
 
         # File menu
         file_menu = QMenu(self.tr("&File"), menu_bar)
@@ -691,9 +709,7 @@ class CanvasMainWindow(QMainWindow):
         self.show_log_action.setChecked(
             settings.value("output-dock/is-visible", False, type=bool))
 
-        default_dir = QDesktopServices.storageLocation(
-            QDesktopServices.DocumentsLocation
-        )
+        default_dir = user_documents_path()
 
         self.last_scheme_dir = settings.value("last-scheme-dir", default_dir,
                                               type=str)
@@ -872,15 +888,14 @@ class CanvasMainWindow(QMainWindow):
 
         if self.last_scheme_dir is None:
             # Get user 'Documents' folder
-            start_dir = QDesktopServices.storageLocation(
-                QDesktopServices.DocumentsLocation)
+            start_dir = user_documents_path()
         else:
             start_dir = self.last_scheme_dir
 
         # TODO: Use a dialog instance and use 'addSidebarUrls' to
         # set one or more extra sidebar locations where Schemes are stored.
         # Also use setHistory
-        filename = QFileDialog.getOpenFileName(
+        filename, _ = QFileDialog.getOpenFileName(
             self, self.tr("Open Orange Workflow File"),
             start_dir, self.tr("Orange Workflow (*.ows)"),
         )
@@ -965,8 +980,8 @@ class CanvasMainWindow(QMainWindow):
             parent=self, env={"basedir": os.path.dirname(filename)})
         errors = []
         try:
-            scheme_load(new_scheme, open(filename, "rb"),
-                        error_handler=errors.append)
+            with open(filename, "rb") as f:
+                scheme_load(new_scheme, f, error_handler=errors.append)
 
         except Exception:
             message_critical(
@@ -1118,7 +1133,8 @@ class CanvasMainWindow(QMainWindow):
         """
         if path and os.path.exists(path):
             try:
-                version = sniff_version(open(path, "rb"))
+                with open(path, "rb") as f:
+                    version = sniff_version(f)
             except (IOError, OSError):
                 log.error("Error opening '%s'", path, exc_info=True)
                 # The client should fail attempting to write.
@@ -1179,13 +1195,11 @@ class CanvasMainWindow(QMainWindow):
             if self.last_scheme_dir is not None:
                 start_dir = self.last_scheme_dir
             else:
-                start_dir = QDesktopServices.storageLocation(
-                    QDesktopServices.DocumentsLocation
-                )
+                start_dir = user_documents_path()
 
             start_dir = os.path.join(str(start_dir), title + ".ows")
 
-        filename = QFileDialog.getSaveFileName(
+        filename, _ = QFileDialog.getSaveFileName(
             self, self.tr("Save Orange Workflow File"),
             start_dir, self.tr("Orange Workflow (*.ows)")
         )
