@@ -9,27 +9,48 @@ import scipy.sparse as sp
 import bottleneck as bn
 
 
+def _count_nans_per_row_sparse(X, weights):
+    """ Count the number of nans (undefined) values per row. """
+    items_per_row = 1 if X.ndim == 1 else X.shape[1]
+    counts = np.ones(X.shape[0]) * items_per_row
+    nnz_per_row = np.bincount(X.indices, minlength=len(counts))
+    counts -= nnz_per_row
+    if weights is not None:
+        counts *= weights
+    return np.sum(counts)
+
+
 def bincount(X, max_val=None, weights=None, minlength=None):
     """Return counts of values in array X.
 
     Works kind of like np.bincount(), except that it also supports floating
     arrays with nans.
     """
+    if sp.issparse(X):
+        minlength = max_val + 1
+        bin_weights = weights[X.indices] if weights is not None else None
+        return (np.bincount(X.data.astype(int),
+                            weights=bin_weights,
+                            minlength=minlength, ),
+                _count_nans_per_row_sparse(X, weights))
+
     X = np.asanyarray(X)
     if X.dtype.kind == 'f' and bn.anynan(X):
         nonnan = ~np.isnan(X)
-        nans = (~nonnan).sum(axis=0)
         X = X[nonnan]
         if weights is not None:
+            nans = (~nonnan * weights).sum(axis=0)
             weights = weights[nonnan]
+        else:
+            nans = (~nonnan).sum(axis=0)
     else:
         nans = 0. if X.ndim == 1 else np.zeros(X.shape[1], dtype=float)
     if minlength is None and max_val is not None:
         minlength = max_val + 1
-    return (np.bincount(X.astype(np.int32, copy=False),
-                        weights=weights,
-                        minlength=minlength).astype(float),
-            nans)
+    bc = np.array([]) if minlength is not None and minlength <= 0 else \
+        np.bincount(X.astype(np.int32, copy=False),
+                    weights=weights, minlength=minlength).astype(float)
+    return bc, nans
 
 
 def countnans(X, weights=None, axis=None, dtype=None, keepdims=False):
@@ -47,13 +68,21 @@ def countnans(X, weights=None, axis=None, dtype=None, keepdims=False):
     -------
     counts
     """
-    X = np.asanyarray(X)
-    isnan = np.isnan(X)
-    if weights is not None and weights.shape == X.shape:
-        isnan = isnan * weights
-    counts = isnan.sum(axis=axis, dtype=dtype, keepdims=keepdims)
-    if weights is not None and weights.shape != X.shape:
-        counts = counts * weights
+    if not sp.issparse(X):
+        X = np.asanyarray(X)
+        isnan = np.isnan(X)
+        if weights is not None and weights.shape == X.shape:
+            isnan = isnan * weights
+        counts = isnan.sum(axis=axis, dtype=dtype, keepdims=keepdims)
+        if weights is not None and weights.shape != X.shape:
+            counts = counts * weights
+    else:
+        if any(attr is not None for attr in [axis, dtype]) or \
+                        keepdims is not False:
+            raise ValueError('Arguments axis, dtype and keepdims'
+                             'are not yet supported on sparse data!')
+
+        counts = _count_nans_per_row_sparse(X, weights)
     return counts
 
 
@@ -192,7 +221,7 @@ def stats(X, weights=None, compute_variance=False):
             X.max(axis=0).toarray().ravel(),
             np.asarray(X.mean(axis=0)).ravel() if not weighted else weighted_mean,
             np.zeros(X.shape[1]),      # variance not supported
-            X.shape[1] - non_zero,
+            X.shape[0] - non_zero,
             non_zero))
     else:
         nans = (~X.astype(bool)).sum(axis=0) if X.size else np.zeros(X.shape[1])
