@@ -5,40 +5,28 @@ import unittest
 
 import numpy as np
 import sklearn.tree as skl_tree
-from sklearn.tree._tree import TREE_LEAF, Tree
+from sklearn.tree._tree import TREE_LEAF
 
 from Orange.data import Table
-from Orange.classification import TreeLearner
-from Orange.regression import TreeRegressionLearner
+from Orange.classification import SklTreeLearner
+from Orange.regression import SklTreeRegressionLearner
 
 
 class TestTreeLearner(unittest.TestCase):
     def test_classification(self):
         table = Table('iris')
-        learn = TreeLearner()
+        learn = SklTreeLearner()
         clf = learn(table)
         Z = clf(table)
         self.assertTrue(np.all(table.Y.flatten() == Z))
 
     def test_regression(self):
         table = Table('housing')
-        learn = TreeRegressionLearner()
+        learn = SklTreeRegressionLearner()
         model = learn(table)
         pred = model(table)
         self.assertTrue(np.all(table.Y.flatten() == pred))
 
-    def test_get_tree_classification(self):
-        table = Table('iris')
-        learn = TreeLearner()
-        clf = learn(table)
-        self.assertIsInstance(clf.tree, Tree)
-
-    def test_get_tree_regression(self):
-        table = Table('housing')
-        learn = TreeRegressionLearner()
-        clf = learn(table)
-        self.assertIsInstance(clf.tree, Tree)
-
 
 class TestDecisionTreeClassifier(unittest.TestCase):
     @classmethod
@@ -115,7 +103,7 @@ class TestDecisionTreeClassifier(unittest.TestCase):
     def test_navigate_tree(self):
         table = self.iris
         clf = skl_tree.DecisionTreeClassifier(max_depth=1)
-        clf = clf.fit(table.X, table.Y)
+        clf = clf.fit(table.X, table.Y.reshape(-1, 1))
         t = clf.tree_
 
         x = table.X[0]
@@ -123,4 +111,4 @@ class TestDecisionTreeClassifier(unittest.TestCase):
             v = t.value[t.children_left[0]][0]
         else:
             v = t.value[t.children_right[0]][0]
-        self.assertEqual(np.argmax(v), clf.predict(table.X[0]))
+        self.assertEqual(np.argmax(v), clf.predict(table.X[:1]))
