@@ -3,8 +3,11 @@ from AnyQt.QtCore import QTimer, Qt
 
 from Orange.classification.base_classification import LearnerClassification
 from Orange.data import Table
+from Orange.modelling import Fitter
 from Orange.preprocess.preprocess import Preprocess
+from Orange.regression.base_regression import LearnerRegression
 from Orange.widgets import gui
+from Orange.widgets import widget
 from Orange.widgets.settings import Setting
 from Orange.widgets.utils.sql import check_sql_input
 from Orange.widgets.widget import OWWidget, WidgetMetaClass, Msg
@@ -21,8 +24,9 @@ class DefaultWidgetChannelsMetaClass(WidgetMetaClass):
         if attrib.get('name', False):
             # Ensure all needed attributes are present
             if not all(attr in attrib for attr in mcls.REQUIRED_ATTRIBUTES):
-                raise AttributeError("'{name}' must have '{attrs}' attributes"
-                                     .format(name=name, attrs="', '".join(mcls.REQUIRED_ATTRIBUTES)))
+                raise AttributeError(
+                    "'{}' must have '{}' attributes"
+                    .format(name, "', '".join(mcls.REQUIRED_ATTRIBUTES)))
 
             attrib['outputs'] = mcls.update_channel(
                 mcls.default_outputs(attrib),
@@ -48,10 +52,11 @@ class DefaultWidgetChannelsMetaClass(WidgetMetaClass):
 
     @classmethod
     def update_channel(cls, channel, items):
-        item_names = set(item[0] for item in channel)
+        item_names = set(item[0] if isinstance(item, tuple) else item.name
+                         for item in channel)
 
         for item in items:
-            if not item[0] in item_names:
+            if item[0] not in item_names:
                 channel.append(item)
 
         return channel
@@ -70,19 +75,26 @@ class OWBaseLearnerMeta(DefaultWidgetChannelsMetaClass):
 
     @classmethod
     def default_inputs(cls, attrib):
-        return [("Data", Table, "set_data"), ("Preprocessor", Preprocess, "set_preprocessor")]
+        return [("Data", Table, "set_data"),
+                ("Preprocessor", Preprocess, "set_preprocessor")]
 
     @classmethod
     def default_outputs(cls, attrib):
         learner_class = attrib['LEARNER']
+        replaces = []
         if issubclass(learner_class, LearnerClassification):
             model_name = 'Classifier'
-        else:
+        elif issubclass(learner_class, LearnerRegression):
             model_name = 'Predictor'
+        else:
+            model_name = 'Model'
+            replaces = ['Classifier', 'Predictor']
 
         attrib['OUTPUT_MODEL_NAME'] = model_name
-        return [("Learner", learner_class),
-                (model_name, attrib['LEARNER'].__returns__)]
+
+        return [widget.OutputSignal("Learner", learner_class),
+                widget.OutputSignal(model_name, learner_class.__returns__,
+                                    replaces=replaces)]
 
     @classmethod
     def add_extra_attributes(cls, name, attrib):
@@ -94,12 +106,13 @@ class OWBaseLearnerMeta(DefaultWidgetChannelsMetaClass):
 class OWBaseLearner(OWWidget, metaclass=OWBaseLearnerMeta):
     """Abstract widget for classification/regression learners.
 
-    Notes:
-        All learner widgets should define learner class LEARNER.
-        LEARNER should have __returns__ attribute.
+    Notes
+    -----
+    All learner widgets should define learner class LEARNER.
+    LEARNER should have __returns__ attribute.
 
-        Overwrite `create_learner`, `add_main_layout` and
-        `get_learner_parameters` in case LEARNER has extra parameters.
+    Overwrite `create_learner`, `add_main_layout` and `get_learner_parameters`
+    in case LEARNER has extra parameters.
 
     """
     LEARNER = None
@@ -110,6 +123,7 @@ class OWBaseLearner(OWWidget, metaclass=OWBaseLearnerMeta):
 
     class Error(OWWidget.Error):
         data_error = Msg("{}")
+        fitting_failed = Msg("Fitting failed.\n{}")
 
     class Warning(OWWidget.Warning):
         outdated_learner = Msg("Press Apply to submit changes.")
@@ -122,6 +136,7 @@ class OWBaseLearner(OWWidget, metaclass=OWBaseLearnerMeta):
         self.model = None
         self.preprocessors = None
         self.outdated_settings = False
+
         self.setup_layout()
         QTimer.singleShot(0, getattr(self, "unconditional_apply", self.apply))
 
@@ -129,7 +144,7 @@ class OWBaseLearner(OWWidget, metaclass=OWBaseLearnerMeta):
         """Creates a learner with current configuration.
 
         Returns:
-            Leaner: an instance of Orange.base.learner subclass.
+            Learner: an instance of Orange.base.learner subclass.
         """
         return self.LEARNER(preprocessors=self.preprocessors)
 
@@ -143,8 +158,7 @@ class OWBaseLearner(OWWidget, metaclass=OWBaseLearnerMeta):
         return []
 
     def set_preprocessor(self, preprocessor):
-        """Add user-set preprocessors before the default, mandatory ones"""
-        self.preprocessors = ((preprocessor,) if preprocessor else ()) + tuple(self.LEARNER.preprocessors)
+        self.preprocessors = preprocessor
         self.apply()
 
     @check_sql_input
@@ -159,26 +173,36 @@ class OWBaseLearner(OWWidget, metaclass=OWBaseLearnerMeta):
         self.update_model()
 
     def apply(self):
-        """Applies leaner and sends new model."""
+        """Applies learner and sends new model."""
         self.update_learner()
         self.update_model()
 
     def update_learner(self):
         self.learner = self.create_learner()
+        if self.learner and issubclass(self.LEARNER, Fitter):
+            self.learner.use_default_preprocessors = True
         if self.learner is not None:
             self.learner.name = self.learner_name
         self.send("Learner", self.learner)
         self.outdated_settings = False
         self.Warning.outdated_learner.clear()
 
+    def show_fitting_failed(self, exc):
+        """Show error when fitting fails.
+            Derived widgets can override this to show more specific messages."""
+        self.Error.fitting_failed(str(exc), shown=exc is not None)
+
     def update_model(self):
+        self.show_fitting_failed(None)
+        self.model = None
         if self.check_data():
-            self.model = self.learner(self.data)
-            self.model.name = self.learner_name
-            self.model.instances = self.data
-            self.valid_data = True
-        else:
-            self.model = None
+            try:
+                self.model = self.learner(self.data)
+            except BaseException as exc:
+                self.show_fitting_failed(exc)
+            else:
+                self.model.name = self.learner_name
+                self.model.instances = self.data
         self.send(self.OUTPUT_MODEL_NAME, self.model)
 
     def check_data(self):
@@ -226,6 +250,19 @@ class OWBaseLearner(OWWidget, metaclass=OWBaseLearnerMeta):
     def setup_layout(self):
         self.add_learner_name_widget()
         self.add_main_layout()
+        # Options specific to target variable type, if supported
+        if issubclass(self.LEARNER, Fitter):
+            # Only add a classification section if the method is overridden
+            if type(self).add_classification_layout is not \
+                    OWBaseLearner.add_classification_layout:
+                classification_box = gui.widgetBox(
+                    self.controlArea, 'Classification')
+                self.add_classification_layout(classification_box)
+            # Only add a regression section if the method is overridden
+            if type(self).add_regression_layout is not \
+                    OWBaseLearner.add_regression_layout:
+                regression_box = gui.widgetBox(self.controlArea, 'Regression')
+                self.add_regression_layout(regression_box)
         self.add_bottom_buttons()
 
     def add_main_layout(self):
@@ -236,6 +273,24 @@ class OWBaseLearner(OWWidget, metaclass=OWBaseLearnerMeta):
         """
         pass
 
+    def add_classification_layout(self, box):
+        """Creates layout for classification specific options.
+
+        If a widget outputs a learner dispatcher, sometimes the classification
+        and regression learners require different options.
+        See `setup_layout()` method for execution order.
+        """
+        pass
+
+    def add_regression_layout(self, box):
+        """Creates layout for regression specific options.
+
+        If a widget outputs a learner dispatcher, sometimes the classification
+        and regression learners require different options.
+        See `setup_layout()` method for execution order.
+        """
+        pass
+
     def add_learner_name_widget(self):
         self.name_line_edit = gui.lineEdit(
             self.controlArea, self, 'learner_name', box='Name',
