@@ -26,7 +26,7 @@ from Orange.widgets.widget import InputSignal
 from Orange.widgets.visualize.owlinearprojection import LegendItem, ScatterPlotItem
 from Orange.widgets.io import FileFormat
 
-from .owscatterplotgraph import HelpEventDelegate
+from Orange.widgets.visualize.owscatterplotgraph import HelpEventDelegate
 
 def selected_index(view):
     """Return the selected integer `index` (row) in the view.
@@ -250,6 +250,8 @@ class OWDistributions(widget.OWWidget):
         self.groupvar_idx = 0
         self._legend.clear()
         self._legend.hide()
+        self.groupvarview.clear()
+        self.cb_prob.clear()
 
     def _setup_smoothing(self):
         if not self.disc_cont and self.var and self.var.is_continuous:
@@ -285,11 +287,12 @@ class OWDistributions(widget.OWWidget):
         if self.var is None:
             return
         if self.disc_cont:
-            data = self.data[:, (self.var, self.cvar) if self.cvar else self.var ]
+            domain = Orange.data.Domain(
+                [self.var, self.cvar] if self.cvar else [self.var])
+            data = Orange.data.Table(domain, data)
             disc = Orange.preprocess.discretize.EqualWidth(n=self.bins[self.smoothing_index])
-            data = Orange.preprocess.Discretize(data, method=disc,
-                                                remove_const=False)
-            self.var = (list(data.domain) + list(data.domain.metas))[0]
+            data = Orange.preprocess.Discretize(method=disc, remove_const=False)(data)
+            self.var = data.domain[0]
         self.set_left_axis_name()
         self.enable_disable_rel_freq()
         if self.cvar:
