@@ -8,7 +8,7 @@ from AnyQt.QtWidgets import (
 from AnyQt.QtGui import QColor, QBrush, QPen, QFontMetrics
 from AnyQt.QtCore import Qt, QPointF, QSizeF, QRectF
 
-from Orange.tree import TreeModel
+from Orange.base import TreeModel, SklModel
 from Orange.widgets.visualize.owtreeviewer2d import \
     GraphicsNode, GraphicsEdge, OWTreeViewer2D
 from Orange.widgets.utils import to_html
@@ -20,6 +20,8 @@ from Orange.widgets import gui, widget
 from Orange.widgets.utils.colorpalette import ContinuousPaletteGenerator
 from Orange.widgets.utils.annotated_data import (create_annotated_table,
                                                  ANNOTATED_DATA_SIGNAL_NAME)
+from Orange.widgets.visualize.utils.tree.skltreeadapter import SklTreeAdapter
+from Orange.widgets.visualize.utils.tree.treeadapter import TreeAdapter
 
 
 class PieChart(QGraphicsRectItem):
@@ -66,20 +68,21 @@ class TreeNode(GraphicsNode):
     # Methods are documented in PyQt documentation
     # pylint: disable=missing-docstring
 
-    def __init__(self, model, node_inst, parent=None):
+    def __init__(self, tree_adapter, node_inst, parent=None):
         super().__init__(parent)
-        self.model = model
+        self.tree_adapter = tree_adapter
+        self.model = self.tree_adapter.model
         self.node_inst = node_inst
 
         fm = QFontMetrics(self.document().defaultFont())
-        attr = node_inst.attr
+        attr = self.tree_adapter.attribute(node_inst)
         self.attr_text_w = fm.width(attr.name if attr else "")
         self.attr_text_h = fm.lineSpacing()
         self.line_descent = fm.descent()
         self._rect = None
 
-        if model.domain.class_var.is_discrete:
-            self.pie = PieChart(node_inst.value, 8, self)
+        if self.model.domain.class_var.is_discrete:
+            self.pie = PieChart(self.tree_adapter.get_distribution(node_inst)[0], 8, self)
         else:
             self.pie = None
 
@@ -90,7 +93,7 @@ class TreeNode(GraphicsNode):
         self.droplet.setPos(self.rect().center().x(), self.rect().height())
         self.droplet.setVisible(bool(self.branches))
         fm = QFontMetrics(self.document().defaultFont())
-        attr = self.node_inst.attr
+        attr = self.tree_adapter.attribute(self.node_inst)
         self.attr_text_w = fm.width(attr.name if attr else "")
         self.attr_text_h = fm.lineSpacing()
         self.line_descent = fm.descent()
@@ -129,7 +132,7 @@ class TreeNode(GraphicsNode):
         font = self.document().defaultFont()
         painter.setFont(font)
         if self.parent:
-            draw_text = self.node_inst.description
+            draw_text = str(self.tree_adapter.short_rule(self.node_inst))
             if self.parent.x() > self.x():  # node is to the left
                 fm = QFontMetrics(font)
                 x = rect.width() / 2 - fm.width(draw_text) - 4
@@ -140,7 +143,7 @@ class TreeNode(GraphicsNode):
         painter.setBrush(self.backgroundBrush)
         painter.setPen(QPen(Qt.black, 3 if self.isSelected() else 0))
         adjrect = rect.adjusted(-3, 0, 0, 0)
-        if not self.node_inst.children:
+        if not self.tree_adapter.has_children(self.node_inst):
             painter.drawRoundedRect(adjrect, 4, 4)
         else:
             painter.drawRect(adjrect)
@@ -155,14 +158,31 @@ class OWTreeGraph(OWTreeViewer2D):
     name = "Tree Viewer"
     icon = "icons/TreeViewer.svg"
     priority = 35
-    inputs = [("Tree", TreeModel, "ctree")]
-    outputs = [("Selected Data", Table, widget.Default),
-               (ANNOTATED_DATA_SIGNAL_NAME, Table)]
+    inputs = [
+        widget.InputSignal(
+            "Tree", TreeModel, "ctree",
+            # Had different input names before merging from
+            # Classification/Regression tree variants
+            replaces=["Classification Tree", "Regression Tree"])
+    ]
+    outputs = [
+        widget.OutputSignal(
+            "Selected Data", Table, widget.Default, id="selected-data",
+        ),
+        widget.OutputSignal(
+            ANNOTATED_DATA_SIGNAL_NAME, Table, id="annotated-data")
+    ]
+
 
     settingsHandler = ClassValuesContextHandler()
     target_class_index = ContextSetting(0)
     regression_colors = Setting(0)
 
+    replaces = [
+        "Orange.widgets.classify.owclassificationtreegraph.OWClassificationTreeGraph",
+        "Orange.widgets.classify.owregressiontreegraph.OWRegressionTreeGraph"
+    ]
+
     COL_OPTIONS = ["Default", "Number of instances", "Mean value", "Variance"]
     COL_DEFAULT, COL_INSTANCE, COL_MEAN, COL_VARIANCE = range(4)
 
@@ -171,6 +191,7 @@ class OWTreeGraph(OWTreeViewer2D):
         self.domain = None
         self.dataset = None
         self.clf_dataset = None
+        self.tree_adapter = None
 
         self.color_label = QLabel("Target class: ")
         combo = self.color_combo = gui.OrangeComboBox()
@@ -194,9 +215,8 @@ class OWTreeGraph(OWTreeViewer2D):
             node.set_rect(QRectF(rect.x(), rect.y(), w, rect.height()))
         self.scene.fix_pos(self.root_node, 10, 10)
 
-    @staticmethod
-    def _update_node_info_attr_name(node, text):
-        attr = node.node_inst.attr
+    def _update_node_info_attr_name(self, node, text):
+        attr = self.tree_adapter.attribute(node.node_inst)
         if attr is not None:
             text += "<hr/>{}".format(attr.name)
         return text
@@ -217,6 +237,7 @@ class OWTreeGraph(OWTreeViewer2D):
         if self.domain.class_var.is_discrete:
             self.target_class_index = i
             self.toggle_node_color_cls()
+            self.set_node_info()
         else:
             self.regression_colors = i
             self.toggle_node_color_reg()
@@ -246,7 +267,9 @@ class OWTreeGraph(OWTreeViewer2D):
             self.info.setText('No tree.')
             self.root_node = None
             self.dataset = None
+            self.tree_adapter = None
         else:
+            self.tree_adapter = self._get_tree_adapter(model)
             self.domain = model.domain
             self.dataset = model.instances
             if self.dataset is not None and self.dataset.domain != self.domain:
@@ -267,41 +290,44 @@ class OWTreeGraph(OWTreeViewer2D):
                 self.color_combo.addItems(self.COL_OPTIONS)
                 self.color_combo.setCurrentIndex(self.regression_colors)
             self.openContext(self.domain.class_var)
-            self.root_node = self.walkcreate(model.root, None)
-            self.info.setText('{} nodes, {} leaves'.
-                              format(model.node_count(), model.leaf_count()))
+            # self.root_node = self.walkcreate(model.root, None)
+            self.root_node = self.walkcreate(self.tree_adapter.root)
+            self.info.setText('{} nodes, {} leaves'.format(
+                self.tree_adapter.num_nodes,
+                len(self.tree_adapter.leaves(self.tree_adapter.root))))
         self.setup_scene()
         self.send("Selected Data", None)
         self.send(ANNOTATED_DATA_SIGNAL_NAME,
                   create_annotated_table(self.dataset, []))
 
-    def walkcreate(self, node_inst, parent=None):
+    def walkcreate(self, node, parent=None):
         """Create a structure of tree nodes from the given model"""
-        node = TreeNode(self.model, node_inst, parent)
-        self.scene.addItem(node)
+        node_obj = TreeNode(self.tree_adapter, node, parent)
+        self.scene.addItem(node_obj)
         if parent:
-            edge = GraphicsEdge(node1=parent, node2=node)
+            edge = GraphicsEdge(node1=parent, node2=node_obj)
             self.scene.addItem(edge)
             parent.graph_add_edge(edge)
-        for child_inst in node_inst.children:
+        for child_inst in self.tree_adapter.children(node):
             if child_inst is not None:
-                self.walkcreate(child_inst, node)
-        return node
+                self.walkcreate(child_inst, node_obj)
+        return node_obj
 
     def node_tooltip(self, node):
-        return "<br>".join(to_html(rule)
-                           for rule in self.model.rule(node.node_inst))
+        return "<br>".join(to_html(str(rule))
+                           for rule in self.tree_adapter.rules(node.node_inst))
 
     def update_selection(self):
         if self.model is None:
             return
         nodes = [item.node_inst for item in self.scene.selectedItems()
                  if isinstance(item, TreeNode)]
-        data = self.model.get_instances(nodes)
+        data = self.tree_adapter.get_instances_in_nodes(
+            self.clf_dataset, nodes)
         self.send("Selected Data", data)
-        self.send(ANNOTATED_DATA_SIGNAL_NAME,
-                  create_annotated_table(self.dataset,
-                                         self.model.get_indices(nodes)))
+        self.send(ANNOTATED_DATA_SIGNAL_NAME, create_annotated_table(
+            self.dataset,
+            self.tree_adapter.get_indices(nodes)))
 
     def send_report(self):
         if not self.model:
@@ -327,8 +353,8 @@ class OWTreeGraph(OWTreeViewer2D):
     def update_node_info_cls(self, node):
         """Update the printed contents of the node for classification trees"""
         node_inst = node.node_inst
-        distr = node_inst.value
-        total = len(node_inst.subset)
+        distr = self.tree_adapter.get_distribution(node_inst)[0]
+        total = self.tree_adapter.num_samples(node_inst)
         distr = distr / np.sum(distr)
         if self.target_class_index:
             tabs = distr[self.target_class_index - 1]
@@ -351,8 +377,8 @@ class OWTreeGraph(OWTreeViewer2D):
     def update_node_info_reg(self, node):
         """Update the printed contents of the node for regression trees"""
         node_inst = node.node_inst
-        mean, var = node_inst.value
-        insts = len(node_inst.subset)
+        mean, var = self.tree_adapter.get_distribution(node_inst)[0]
+        insts = self.tree_adapter.num_samples(node_inst)
         text = "{:.1f} ± {:.1f}<br/>".format(mean, var)
         text += "{} instances".format(insts)
         text = self._update_node_info_attr_name(node, text)
@@ -363,7 +389,7 @@ class OWTreeGraph(OWTreeViewer2D):
         """Update the node color for classification trees"""
         colors = self.scene.colors
         for node in self.scene.nodes():
-            distr = node.node_inst.value
+            distr = node.tree_adapter.get_distribution(node.node_inst)[0]
             total = sum(distr)
             if self.target_class_index:
                 p = distr[self.target_class_index - 1] / total
@@ -384,39 +410,49 @@ class OWTreeGraph(OWTreeViewer2D):
             for node in self.scene.nodes():
                 node.backgroundBrush = brush
         elif self.regression_colors == self.COL_INSTANCE:
-            max_insts = len(self.model.instances)
+            max_insts = len(self.tree_adapter.get_instances_in_nodes(
+                self.dataset, [self.tree_adapter.root]))
             for node in self.scene.nodes():
+                node_insts = len(self.tree_adapter.get_instances_in_nodes(
+                    self.dataset, [node.node_inst]))
                 node.backgroundBrush = QBrush(def_color.lighter(
-                    120 - 20 * len(node.node_inst.subset) / max_insts))
+                    120 - 20 * node_insts / max_insts))
         elif self.regression_colors == self.COL_MEAN:
             minv = np.nanmin(self.dataset.Y)
             maxv = np.nanmax(self.dataset.Y)
             fact = 1 / (maxv - minv) if minv != maxv else 1
             colors = self.scene.colors
             for node in self.scene.nodes():
-                node.backgroundBrush = QBrush(
-                    colors[fact * (node.node_inst.value[0] - minv)])
+                node_mean = self.tree_adapter.get_distribution(node.node_inst)[0][0]
+                node.backgroundBrush = QBrush(colors[fact * (node_mean - minv)])
         else:
             nodes = list(self.scene.nodes())
-            variances = [node.node_inst.value[1] for node in nodes]
+            variances = [self.tree_adapter.get_distribution(node.node_inst)[0][1]
+                         for node in nodes]
             max_var = max(variances)
             for node, var in zip(nodes, variances):
                 node.backgroundBrush = QBrush(def_color.lighter(
                     120 - 20 * var / max_var))
         self.scene.update()
 
+    def _get_tree_adapter(self, model):
+        if isinstance(model, SklModel):
+            return SklTreeAdapter(model)
+        return TreeAdapter(model)
+
 
 def test():
     """Standalone test"""
     import sys
     from AnyQt.QtWidgets import QApplication
-#    from Orange.classification.tree import TreeLearner
-    from Orange.regression.tree import TreeLearner
+    from Orange.classification.tree import TreeLearner, SklTreeLearner
+    from Orange.regression.tree import TreeLearner, SklTreeRegressionLearner
     a = QApplication(sys.argv)
     ow = OWTreeGraph()
-    # data = Table("iris")
-    data = Table("housing")
-    clf = TreeLearner()(data)
+    data = Table("titanic")
+    # data = Table("housing")[:30]
+    clf = SklTreeLearner()(data)
+    # clf = TreeLearner()(data)
     clf.instances = data
 
     ow.ctree(clf)
